<?php

/**
 * Search service class using Fusion server.
 */
class SarniaFusionService extends SarniaSolrService {

  /**
   * Use a connection class tailored to Fusion.
   */
  protected $connection_class = 'SearchApiFusionConnection';

  /**
   * {@inheritdoc}
   */
  public function configurationForm(array $form, array &$form_state) {
    // Get sarnia form.
    $form = parent::configurationForm($form, $form_state);

    // Change field labels and default values (from Solr to Fusion).
    if ($this->options) {
      $form['server_description']['#title'] = t('Fusion server URI');
    }

    $options = $this->options + array(
      'port' => '8764',
      'retrieve_data' => TRUE,
      'skip_schema_check' => TRUE,
      'fusion_app' => '',
      'fusion_qprofile_search' => '',
      'fusion_qprofile_autocomplete' => '',
      'fusion_excerpt_field' => '',
    );

    $form['host']['#title'] = t('Fusion host');
    $form['host']['#description'] = t('The host name or IP of your Fusion server, e.g. <code>localhost</code> or <code>www.example.com</code>.');

    $form['port']['#title'] = t('Fusion port');
    $form['port']['#description'] = t('By default the Fusion UI runs on port 8764.');
    $form['port']['#default_value'] = $options['port'];

    $form['advanced']['retrieve_data']['#default_value'] = $options['retrieve_data'];
    $form['advanced']['skip_schema_check']['#default_value'] = $options['skip_schema_check'];

    // Make the path static and set it to /api.
    $form['path'] = array(
      '#type' => 'value',
      '#value' => '/api',
    );

    // Add extra fields specific to Fusion.
    $form['fusion_app'] = array(
      '#type' => 'textfield',
      '#title' => t('Fusion app'),
      '#description' => t('The name of the Fusion app.'),
      '#default_value' => $options['fusion_app'],
      '#required' => TRUE,
    );
    $form['fusion_qprofile_search'] = array(
      '#type' => 'textfield',
      '#title' => t('Fusion search query profile'),
      '#description' => t('The Fusion query profile to use when searching.'),
      '#default_value' => $options['fusion_qprofile_search'],
      '#required' => TRUE,
    );
    if (module_exists('search_api_autocomplete')) {
      $form['fusion_qprofile_autocomplete'] = array(
        '#type' => 'textfield',
        '#title' => t('Fusion autocomplete query profile'),
        '#description' => t('The Fusion query profile to use when generating autocomplete suggestions.'),
        '#default_value' => $options['fusion_qprofile_autocomplete'],
        '#required' => TRUE,
      );
    }
    $form['fusion_excerpt_field'] = array(
      '#type' => 'textfield',
      '#title' => t('Field used for excerpts'),
      '#description' => t('The field containing the search result excerpts.'),
      '#default_value' => $options['fusion_excerpt_field'],
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getExtraInformation() {
    // Show sarnia information.
    $info = parent::getExtraInformation();

    // Show extra fields specific to Fusion.
    $info[] = array(
      'label' => t('Fusion app'),
      'info' => $this->options['fusion_app'],
    );
    $info[] = array(
      'label' => t('Fusion search query profile'),
      'info' => $this->options['fusion_qprofile_search'],
    );
    $info[] = array(
      'label' => t('Fusion autocomplete query profile'),
      'info' => $this->options['fusion_qprofile_autocomplete'],
    );

    return $info;
  }

  /**
   * {@inheritdoc}
   */
  protected function preQuery(array &$call_args, SearchApiQueryInterface $query) {
    // Set the default query and request handler via sarnia.
    parent::preQuery($call_args, $query);

    // Remove qf parameter set by SearchApiSolrConnection as we don't need this when querying Fusion.
    unset($call_args['params']['qf']);

    // Highlighting is also configured on Fusion itself so there is no need to send hl parameters.
    foreach ($call_args['params'] as $param => $value) {
      if (substr($param, 0, strlen('hl')) === 'hl') {
        unset($call_args['params'][$param]);
      }
    }

    // Spellchecker is also configured on Fusion itself.
    unset($call_args['params']['spellcheck']);

    // Sarnia sets the fl parameter to '*' which causes all fields to be loaded on Fusion and is bad for performance.
    // Hence we remove it.
    // As a result if you build a view based on Fusion you have to use the fields returned by default by Fusion.
    unset($call_args['params']['fl']);
  }

  /**
   * {@inheritdoc}
   */
  protected function extractResults(SearchApiQueryInterface $query, $response) {
    $results = parent::extractResults($query, $response);

    // Extract landing pages.
    if (!empty($response->fusion->{'landing-pages'})) {
      $landing_pages = $response->fusion->{'landing-pages'};
      foreach ($landing_pages as $landing_page) {
        list($url, $title) = explode('$$$', $landing_page);
        $results['fusion_landing_pages'][$url] = $title;
      }
    }

    // Fusion outputs spellcheck suggestions in a slightly different way than Solr so we use a custom class.
    if (module_exists('search_api_spellcheck') && $query->getOption('search_api_spellcheck')) {
      $results['search_api_spellcheck'] = new SearchApiSpellcheckFusion($response);
    }

    return $results;
  }

  /**
   * {@inheritdoc}
   */
  public function getAutocompleteSuggestions(SearchApiQueryInterface $query, SearchApiAutocompleteSearch $search, $incomplete_key, $user_input) {
    $suggestions = array();
    
    try {
      $this->connect();
      $suggestions = $this->solr->getAutocompleteSuggestions($user_input);
    }
    catch (SearchApiException $e) {
      watchdog_exception('search_api_fusion', $e, '%type during autocomplete Fusion query: !message in %function (line %line of %file).', array(), WATCHDOG_WARNING);
    }

    return $suggestions;
  }

  /**
   * {@inheritdoc}
   */
  protected function getExcerpt($response, $id, array &$fields, array $field_mapping, array $fulltext_fields) {
    // Extract excerpts, if a field was provided. This is used by search_api_page.
    // TODO turn into a drop-down list instead of a text field.
    if (!empty($this->options['fusion_excerpt_field'])) {
      $excerpt_field = $this->options['fusion_excerpt_field'];
      if (isset($fields[$excerpt_field])) {
        return $fields[$excerpt_field];
      }
    }
  }

}
